#define MAIN_PROGRAM
#define	BP_DEBUG
#include <stdio.h>
#include <modes.h>
#include <balboa.h>
#include <bp_status.h>
#include <bp_error.h>
#include <bp_mem.h>
#include <vm_video.h>
#include <vm_dcp.h>
#include <vm_vs.h>
#include <vm_txt.h>
#include <sgm.h>
#include <cursor.h>
#include <controls.h>
#include <bam.h>
#include <ucm.h>
#include <vm_picture.h>
#include <vm_clut.h>
#include <cdfm.h>
#include <csd.h>
#include <signal.h>
#include <iff.h>
#include <iff_aiff.h>

VS		*VideoScreen;
PICTURE	*Picture;
HOTSPOT	*HotSpot;
int		interlace_setting;
int		vres_setting;

void exit_balboa();
void force_exit();
void root_func();

CPCALLBACK	root_cb	= {EV_B0D | EV_B1D | DISP_LASTONE, root_func, 0};

int init_balboa(argc, argv)
	int		argc;
	char	*argv[];
{
	balboa_init("/pcd0h/CMDS", 0);

	sgm_enable(SIGINT, force_exit, 0, IMMEDIATE);
	sgm_enable(SIGQUIT, force_exit, 0, IMMEDIATE);

    dispatch_atquit(exit_balboa, NULL);

	STATUS_INIT(stdout, ST_MGR_ALL & ~(ST_TMR_MGR | ST_ANM_MGR), ST_TYP_ALL & ~ST_ITERATE, 50);

	init_video("title.iff");
	init_cursor();

	printf("Running!\n");

    return	OK;
}

void root_func(context, msg)
    int             context;
    register CURSOR_STATE *msg;
{
	register	unsigned char	*p;
	register	int				x;
	register	int				y;


	/* Both buttons at once is exit */
	if (msg->event & EV_B0D)
	{
		if (msg->event & EV_B1D)
		{
			dispatch_quit(0);
		}

		x	= msg->world_x;
		y	= msg->world_y;

		printf("World:(%d, %d); UCM:(%d, %d)\n", x, y, msg->ucm.x, msg->ucm.y);

		switch (Picture->pi_coding & D_TYPE)
		{
			case D_CLUT8:
			case D_CLUT7:
				p	= Picture->pi_pstart + (Picture->pi_pwidth * (y >> 1)) + (x >> 1);

				printf("Color entry is %d\n", *p);

				break;

			case D_CLUT4:
				p = Picture->pi_pstart + (Picture->pi_pwidth * (y >> 1)) + (x >> 1);
				printf("Color entry is %d\n", (x & 1) ? (*p & 0xFF) : ((*p >> 4) & 0xFF));

				break;
		}
	}

    else if (msg->event & EV_B1D)
	{
		/* Place our picture at the front */
		if (pi_front(VideoScreen, Picture) == SYSERR)
		{
			STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
				"init_video: pi_front() failed to place picture");

			return;
		}

		/* Update the LCTs for this picture */
		if (vs_update(VideoScreen, interlace_setting, vm_display_type, NULL, NULL) == SYSERR)
		{
			STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
				"init_video: vs_update() failed to update LCTs");

			return;
		}
	}
}

void FIFOFullFunc()
{
    STATUS(ST_APPL, ST_INFO_STR, 0, "Dispatcher FIFO is 90% full");
}

int main(argc, argv)
	int		argc;
	char	*argv[];
{
    interlace_setting	= 0;
    vres_setting		= 0;
	errno				= 0;

    sgm_init();

	dispatch_init(75, BP_MEM_DONTCARE, 60, FIFOFullFunc);
	dispatch_loop(init_balboa, argc, argv);
}

void force_exit(sig, param)
	short sig;
	int param;
{
	if (sig == SIGINT)
	{
		printf("\n^C -- ignored. Use ^E to exit\n");
	}

	else
	{
	    dispatch_quit(errno);
	}
}

void exit_balboa()
{
	/* Shutdown video & Cursor */
	vs_close(VideoScreen);
    vs_finish();

	if (errno != 0)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "Balboa: Bad Exit");
	}

	else
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_INFO_STR, 0, "Balboa: Good Exit");
	}

    exit(0);
}

int init_video(filename)
    register char  *filename;
{
    STATUS(ST_APPL, ST_TYP_APP2 | ST_INFO_STR, 0, "Balboa: Initialising Video");

	/* Initialize the video manager with two video large buffers */
	if (vs_init(BP_MEM_PLANEA, 2) == SYSERR)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_init() failed");

		dispatch_quit(SYSERR);
	}

	if (-1 == vi_init(VideoScreen->vs_videnv, BP_MEM_PLANEA, 2, 2))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "vi_init failed");

		dispatch_quit(SYSERR);
	}

	/* Open a video screen for LCT double buffering and Cursor On */
	if (!(VideoScreen = vs_open(LCT_DOUBLE | CURSOR_ON, 3, BP_MEM_PLANEB)))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_open() failed");

		dispatch_quit(SYSERR);
	}

	/* Initialize the CLUT manager for this Video Screen */
	if (cl_init(VideoScreen, BP_MEM_DONTCARE, 0) == SYSERR)
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: cl_init() failed to initialize CLUT manager");

		dispatch_quit(SYSERR);
	}

	/* Input our IFF image file */
	if (!(Picture = pi_iffpic(PLANE_A, vres_setting, filename, BP_MEM_DONTCARE)))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: pi_iffpic() failed to load file");

		dispatch_quit(SYSERR);
	}

	/* Take care of any associated CLUT */
	if (Picture->pi_clut)
	{
		if (SYSERR == cl_exec(VideoScreen, PLANE_A, Picture->pi_clut))
		{
			STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
				"init_video: Unable to write picture's CLUT");

			dispatch_quit(SYSERR);
		}
    }

	/* Place our picture at the front */
	if (SYSERR == pi_front(VideoScreen, Picture))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
			"init_video: pi_front() failed to place picture");

		dispatch_quit(SYSERR);
	}

	/* Update the LCTs for this picture */
	if (SYSERR == vs_update(VideoScreen, interlace_setting, vm_display_type, NULL, NULL))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
			"init_video: vs_update() failed to update LCTs");

		dispatch_quit(SYSERR);
	}

	/* Display the Video Screen (Activate FCTs and cursor) */
	if (SYSERR == vs_show(VideoScreen))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno, "init_video: vs_show() failed");

		dispatch_quit(SYSERR);
	}

	return	OK;
}

int init_cursor()
{
	if (!(HotSpot = hot_create_root(vm_display_hsize, vm_display_vsize, NULL, root_cb, NULL,
		BP_MEM_DONTCARE)))
	{
		STATUS(ST_APPL, ST_TYP_APP2 | ST_ERROR, errno,
			"init_cursor: Unable to create hotspot tree root");

		return	SYSERR;
	}

	vs_cp_hstree_install(VideoScreen, HotSpot, 0, 0, BP_MEM_DONTCARE);

	/* Enable cursor Callback */
	vs_cp_start(VideoScreen);
	vs_cp_show(VideoScreen);
	vs_cp_start(VideoScreen);

	return	OK;
}

